Attribute VB_Name = "FeatureUtils"
Option Explicit
Const c_sModuleFileName As String = "FeatureUtils.bas"
Public Function CreatePolyLineFeatureClass(strFolder As String, strName As String, ByRef spatRef As ISpatialReference, fieldsDesc As SDMUtilities.SDMList) As IFeatureClass

5:   Const strShapeFieldName As String = "Shape"
  
  ' Open the folder to contain the shapefile as a workspace
    Dim pFWS As IFeatureWorkspace
    Dim pWorkspaceFactory As IWorkspaceFactory
10:   Set pWorkspaceFactory = New ShapefileWorkspaceFactory
11:   Set pFWS = pWorkspaceFactory.OpenFromFile(strFolder, 0)
  
  ' Set up a simple fields collection
    Dim pfields As esrigeodatabase.IFields
    Dim pFieldsEdit As esrigeodatabase.IFieldsEdit
16:   Set pfields = New esrigeodatabase.Fields
17:   Set pFieldsEdit = pfields
  
    Dim pField As esrigeodatabase.IField
    Dim pFieldEdit As esrigeodatabase.IFieldEdit
  
  ' Make the shape field
  ' it will need a geometry definition, with a spatial reference
24:   Set pField = New esrigeodatabase.Field
25:   Set pFieldEdit = pField
26:   pFieldEdit.Name = strShapeFieldName
27:   pFieldEdit.Type = esriFieldTypeGeometry
  
  
  Dim pGeomDef As IGeometryDef
  Dim pGeomDefEdit As IGeometryDefEdit
32:   Set pGeomDef = New GeometryDef
33:   Set pGeomDefEdit = pGeomDef
34:   With pGeomDefEdit
35:     .GeometryType = esriGeometryPolyline
36:     Set .SpatialReference = spatRef
37:   End With
38:   Set pFieldEdit.GeometryDef = pGeomDef
39:   pFieldsEdit.AddField pField

  ' Add another miscellaneous text field
  Dim fieldDef As Variant
43:   For Each fieldDef In fieldsDesc.list
    Dim ub As Long
45:     Set pField = New esrigeodatabase.Field
46:     Set pFieldEdit = pField
47:     With pFieldEdit
48:         .Length = CLng(fieldDef(2))
49:         .Name = fieldDef(0)
50:         .Type = fieldDef(1)
51:         .AliasName = fieldDef(3)
52:     End With
53:     pFieldsEdit.AddField pField
    
55:   Next fieldDef
  
  ' Create the shapefile
  ' (some parameters apply to geodatabase options and can be defaulted as Nothing)
    Dim pFeatClass As IFeatureClass
60:   Set pFeatClass = pFWS.CreateFeatureClass(strName, pfields, Nothing, _
                                           Nothing, esriFTSimple, strShapeFieldName, "")
62:   Set CreatePolyLineFeatureClass = pFeatClass
End Function


Public Function CreatePointFeatureClass(strFolder As String, strName As String, ByRef spatRef As ISpatialReference, fieldsDesc As SDMUtilities.SDMList) As IFeatureClass

'  Const strFolder As String = "D:\Data"
'  Const strName As String = "MyShapeFile" ' Dont include .shp extension
70:   Const strShapeFieldName As String = "Shape"
  
  ' Open the folder to contain the shapefile as a workspace
  Dim pFWS As IFeatureWorkspace
  Dim pWorkspaceFactory As IWorkspaceFactory
75:   Set pWorkspaceFactory = New ShapefileWorkspaceFactory
76:   Set pFWS = pWorkspaceFactory.OpenFromFile(strFolder, 0)
  
  ' Set up a simple fields collection
  Dim pfields As esrigeodatabase.IFields
  Dim pFieldsEdit As esrigeodatabase.IFieldsEdit
81:   Set pfields = New esrigeodatabase.Fields
82:   Set pFieldsEdit = pfields
  
  Dim pField As esrigeodatabase.IField
  Dim pFieldEdit As esrigeodatabase.IFieldEdit
  
  ' Make the shape field
  ' it will need a geometry definition, with a spatial reference
89:   Set pField = New esrigeodatabase.Field
90:   Set pFieldEdit = pField
91:   pFieldEdit.Name = strShapeFieldName
92:   pFieldEdit.Type = esriFieldTypeGeometry
  
  
  Dim pGeomDef As IGeometryDef
  Dim pGeomDefEdit As IGeometryDefEdit
97:   Set pGeomDef = New GeometryDef
98:   Set pGeomDefEdit = pGeomDef
99:   With pGeomDefEdit
100:     .GeometryType = esriGeometryPoint
101:     Set .SpatialReference = spatRef
102:   End With
103:   Set pFieldEdit.GeometryDef = pGeomDef
104:   pFieldsEdit.AddField pField

  ' Add another miscellaneous text field
  Dim fieldDef As Variant
108:   For Each fieldDef In fieldsDesc.list
    Dim ub As Long
110:     Set pField = New esrigeodatabase.Field
111:     Set pFieldEdit = pField
112:     With pFieldEdit
113:         .Length = CLng(fieldDef(2))
114:         .Name = fieldDef(0)
115:         .Type = fieldDef(1)
116:         .AliasName = fieldDef(3)
117:     End With
118:     pFieldsEdit.AddField pField
    
120:   Next fieldDef
  
  ' Create the shapefile
  ' (some parameters apply to geodatabase options and can be defaulted as Nothing)
  Dim pFeatClass As IFeatureClass
125:   Set pFeatClass = pFWS.CreateFeatureClass(strName, pfields, Nothing, _
                                           Nothing, esriFTSimple, strShapeFieldName, "")
127:   Set CreatePointFeatureClass = pFeatClass
End Function

Function startEditing(pWorkSpaceName As String) As IWorkspaceEdit
  On Error GoTo ErrorHandler

        Dim pWorkspaceFactory As IWorkspaceFactory
        Dim pWorkspace As IWorkspace
   
136:    Set pWorkspaceFactory = New ShapefileWorkspaceFactory
137:    Set pWorkspace = pWorkspaceFactory.OpenFromFile(pWorkSpaceName, 0)
  
        Dim pWorkspaceEdit As IWorkspaceEdit
140:    Set pWorkspaceEdit = pWorkspace
  
142:    pWorkspaceEdit.StopEditOperation
143:    pWorkspaceEdit.StopEditing False
144:    pWorkspaceEdit.startEditing True
145:    pWorkspaceEdit.StartEditOperation
  
147:    Set startEditing = pWorkspaceEdit


  Exit Function
ErrorHandler:
  HandleError True, "startEditing " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
153:   Set startEditing = Nothing
End Function

Sub StopEditing(ByRef pWorkspaceEdit As IWorkspaceEdit)
157:   pWorkspaceEdit.StopEditOperation
158:   pWorkspaceEdit.StopEditing True
End Sub


















