Attribute VB_Name = "RasterAEUtils"
Option Explicit
Const c_sModuleFileName As String = "RasterAEUtils"

Sub SetToSDMRAEnv(ByRef pEnv As IRasterAnalysisEnvironment)
5:         If Not SDMGlobals.SDMRAEnvironment Is Nothing Then
6:             Set pEnv.Mask = SDMGlobals.SDMRAEnvironment.Mask
                Dim cellsize As Double
8:              SDMGlobals.SDMRAEnvironment.GetCellSize esriRasterEnvValue, cellsize
9:              pEnv.SetCellSize esriRasterEnvValue, cellsize
                Dim raext As IEnvelope
11:             SDMGlobals.SDMRAEnvironment.GetExtent esriRasterEnvValue, raext
12:             pEnv.SetExtent esriRasterEnvValue, raext
13:         Else
                Dim gSDIGetMask As AnalysisParameters.gSDI_getMask
15:             Set gSDIGetMask = New AnalysisParameters.gSDI_getMask
                Dim saLyr As IRasterLayer
17:             Set saLyr = gSDIGetMask.gSDI_getMask
                Dim saMask As IGeoDataset
19:             Set saMask = RasterUtils.GetDatasetRaster(saLyr)
20:             Set pEnv.Mask = saMask
                Dim rasprops As IRasterProps
22:             Set rasprops = RasterUtils.GetDatasetRaster(saLyr)
23:             pEnv.SetCellSize esriRasterEnvValue, rasprops.MeanCellSize.X
24:             pEnv.SetExtent esriRasterEnvValue, saMask.Extent
25:         End If
26:     Set pEnv.OutSpatialReference = SDMGlobals.SDMMap.SpatialReference
27:     Set pEnv.OutWorkspace = SDMGlobals.SDMWorkspace
End Sub

Function getRasterAnalysisEnvironment() As IRasterAnalysisEnvironment
  On Error GoTo ErrorHandler

33:      If Not SDMGlobals.SDMRAEnvironment Is Nothing Then
34:         Set getRasterAnalysisEnvironment = SDMGlobals.SDMRAEnvironment
            Exit Function
36:      End If
         Dim gSDIGetMask As AnalysisParameters.gSDI_getMask
38:      Set gSDIGetMask = New AnalysisParameters.gSDI_getMask
    
        Dim ae As IRasterAnalysisEnvironment
41:     Set ae = New RasterAnalysis
        Dim saLyr As IRasterLayer
43:     Set saLyr = gSDIGetMask.gSDI_getMask
44:         If saLyr Is Nothing Then
45:             Set getRasterAnalysisEnvironment = Nothing
                Exit Function
47:         End If
        Dim saMask As IGeoDataset
49:     Set saMask = RasterUtils.GetDatasetRaster(saLyr)
50:     Set ae.Mask = saMask
        Dim rasprops As IRasterProps
52:     Set rasprops = RasterUtils.GetDatasetRaster(saLyr)
53:     ae.SetCellSize esriRasterEnvValue, rasprops.MeanCellSize.X
54:     ae.SetExtent esriRasterEnvValue, saMask.Extent
55:     Set ae.OutSpatialReference = SDMGlobals.SDMMap.SpatialReference
56:     Set ae.OutWorkspace = SDMGlobals.SDMWorkspace
57:     Set getRasterAnalysisEnvironment = ae


  Exit Function
ErrorHandler:
  HandleError True, "getRasterAnalysisEnvironment " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function

Public Function Combine(ByRef ras1 As IRasterLayer, ByRef ras2 As IRasterLayer) As IGeoDataset
  On Error GoTo ErrorHandler

' Create the RasterLocalOp object
    Dim pLocalOp As ILocalOp
70: Set pLocalOp = New RasterLocalOp
67: Dim pLocOpEnv As IRasterAnalysisEnvironment
72: Set pLocOpEnv = pLocalOp
73: pLocOpEnv.DefaultOutputRasterPrefix = "SDMCompare"
74: Set pLocOpEnv.OutWorkspace = SDMGlobals.SDMWorkspace

' Declare the first input rasterbands
    Dim pInputRaster1 As IRasterBandCollection

' QI for ras1
80: Set pInputRaster1 = RasterUtils.GetDatasetRaster(ras1)

' Declare the second input rasterbands
    Dim pInputRaster2 As IRasterBandCollection

' QI for ras2
86: Set pInputRaster2 = RasterUtils.GetDatasetRaster(ras2)

' Create a raster bandcollection
    Dim pRBCollection As IRasterBandCollection
90: Set pRBCollection = New Raster
91: pRBCollection.AppendBand pInputRaster1.item(0)
92: pRBCollection.AppendBand pInputRaster2.item(0)
    Dim pInputDataset As IGeoDataset
94: Set pInputDataset = pRBCollection

' Declare the output raster object
    Dim pOutputRaster As IGeoDataset

' Calls the method
100: Set pOutputRaster = pLocalOp.Combine(pInputDataset)

102: Set Combine = pOutputRaster

Exit Function
ErrorHandler:
  HandleError True, "Combine " & "RasterAEUtils" & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
107:   Set Combine = Nothing
End Function

Function MakeReclassRaster(ByRef GenTable As ITable, ByRef raslyr As IRasterLayer, genTableName As String, ByRef raMask As IGeoDataset) As IRasterLayer
  On Error GoTo ErrorHandler
'Get generalization table
'Set Remap values
         Dim classIdx As Long
115:     classIdx = GenTable.FindField("Value2")
116:     If classIdx = -1 Then
117:            MsgBox "Value2 field not found in GenTable: " & genTableName & _
                       "Gen Table may not match Evidence attribute table.", vbCritical, "MakeReclassRaster"
119:            Set MakeReclassRaster = Nothing
                Exit Function
121:     End If
         Dim rmap As IRemap
123:     Set rmap = GetRemapValues(GenTable, classIdx)
124:     If rmap Is Nothing Then
125:               MsgBox "Generalization Table: " & genTableName & _
                     "Evidence layer could contain NoData areas.", vbInformation, c_sModuleFileName
127:          Set MakeReclassRaster = Nothing
              Exit Function
129:     End If
         
'Do reclass
         Dim pRasBands As IRasterBandCollection
133:     Set pRasBands = RasterUtils.GetDatasetRaster(raslyr)
         Dim pInputRaster2 As IRasterBand
135:     Set pInputRaster2 = pRasBands.item(0)
         Dim recls As IReclassOp
137:     Set recls = New RasterReclassOp
         Dim reclsopEnv As IRasterAnalysisEnvironment
139:     Set reclsopEnv = recls
140:     Set reclsopEnv.Mask = raMask
141:     reclsopEnv.SetCellSize esriRasterEnvValue, pInputRaster2
142:     reclsopEnv.SetExtent esriRasterEnvValue, raMask
143:     Set reclsopEnv.OutSpatialReference = SDMGlobals.SDMMap.SpatialReference
144:     Set reclsopEnv.OutWorkspace = SDMGlobals.SDMRAEnvironment.OutWorkspace
         Dim newGrid As IGeoDataset
146:     Set newGrid = recls.ReclassByRemap(pInputRaster2, rmap, True)
'Make raster layer from new Grid
         Dim newraslyr As IRasterLayer
149:     Set newraslyr = New RasterLayer
150:     newraslyr.CreateFromRaster newGrid
'     newraslyr.Name = raslyr.Name & "-" & genTableName & "-RCls"
152:     Set MakeReclassRaster = newraslyr
153:     ProgressAnimation.Hide
  Exit Function
ErrorHandler:
  HandleError True, "MakeReclassRaster " & "RasterAEUtils" & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
157:   Set MakeReclassRaster = Nothing
End Function
Public Function GetRemapValues(GenTable As ITable, Value2classIdx As Long) As INumberRemap
  On Error GoTo ErrorHandler
  
162:   If Value2classIdx = -1 Then
163:     Set GetRemapValues = Nothing
164:     MsgBox "Value2 field not found in Gen Table", , c_sModuleFileName
         Exit Function
166:   End If

168:   Set GetRemapValues = New NumberRemap
       Dim curs As ICursor
170:   Set curs = GenTable.Search(Nothing, False)
       Dim row As irow
172:   Set row = curs.NextRow
173:   While Not row Is Nothing
             Dim val As Long
175:         val = row.Value(GenTable.FindField("Value"))
             Dim map As Variant
177:         map = row.Value(Value2classIdx)
178:         If Not IsNull(map) Then
179:            GetRemapValues.MapValue val, CLng(map)
180:         Else
181:            GetRemapValues.MapValueToNoData val
182:         End If
183:         Set row = curs.NextRow
184:   Wend
  Exit Function
ErrorHandler:
  HandleError True, "GetRemapValues " & "RasterAEUtils" & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
188:   Set GetRemapValues = Nothing
End Function

'
'   creates a layer file at the given location for the designated layer
'
Public Sub ExportLayerToPath(pLayer As ILayer, sPath As String)

    On Error GoTo ExportLayerToPath_ERR
    
    Dim pGXLayer As IGxLayer
    Dim pGXFile As IGxFile
    
201:     Set pGXLayer = New GxLayer
202:     Set pGXFile = pGXLayer
    
204:     pGXFile.path = sPath
    
206:     Set pGXLayer.Layer = pLayer
207:     Set pGXFile = Nothing
208:     Set pGXLayer = Nothing

    Exit Sub
    
ExportLayerToPath_ERR:
  HandleError True, "ExportLayerToPath " & "RasterAEUtils" & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
    
End Sub

Public Sub DefineFromList(ByRef pCbx As Variant, ByRef pList As SDMList)
  On Error GoTo ErrorHandler
219:    pCbx.Clear
        Dim i As Long
        Dim pName As Variant
222:     For Each pName In pList.list
223:         pCbx.AddItem CStr(pName)
224:     Next pName


  Exit Sub
ErrorHandler:
  HandleError True, "DefineFromList " & "RasterAEUtil" & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Sub

Public Sub DefineFromLayerList(ByRef pCbx As Variant, ByRef pList As SDMList)
  On Error GoTo ErrorHandler

    Dim i As Long
    Dim plyr As Variant
237:     For Each plyr In pList.list
            Dim lyr As ILayer
239:             Set lyr = plyr
240:         pCbx.AddItem CStr(lyr.Name)
241:     Next plyr


  Exit Sub
ErrorHandler:
  HandleError True, "DefineFromList " & "RasterAEUtil" & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Sub

Function list2String(ByRef lst As SDMList, Optional spacer As String = "") As String
    Dim sublst As SDMList
    Dim item As Variant
252:     For Each item In lst.list
253:       If TypeOf item Is SDMList Then
254:         Set sublst = item
255:         list2String = list2String & vbCrLf & "_" & vbCrLf & list2String(sublst)
256:       Else
257:         list2String = list2String & vbCrLf & CStr(item)
258:       End If
259:     Next item
    
End Function

'''Sub GetRAEfromSAToolbar(pOpEnv As IRasterAnalysisEnvironment)
'''
'''  '5.0 Get environment from toolbar
'''  Dim pTBEnv As IRasterAnalysisEnvironment
'''  Set pTBEnv = New RasterSettings
'''
'''  Dim cs As Double
'''  cs = 0
'''  Dim enumCellSize As esriRasterEnvSettingEnum
'''  pTBEnv.GetCellSize enumCellSize, cs
'''  pOpEnv.SetCellSize enumCellSize, cs
'''
'''  Dim ext As Envelope
'''  Dim enumExtent As esriRasterEnvSettingEnum
'''  pTBEnv.GetExtent enumExtent, ext
'''  pOpEnv.SetExtent enumExtent, ext
'''
'''  Set pOpEnv.Mask = pTBEnv.Mask
'''  Set pOpEnv.OutSpatialReference = pTBEnv.OutSpatialReference
'''  Set pOpEnv.OutWorkspace = pTBEnv.OutWorkspace
'''
'''  pOpEnv.DefaultOutputRasterPrefix = pTBEnv.DefaultOutputRasterPrefix
'''  pOpEnv.DefaultOutputVectorPrefix = pTBEnv.DefaultOutputVectorPrefix
'''  pOpEnv.VerifyType = pTBEnv.VerifyType
'''End Sub

