VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "SpatialAnalystUtil"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit
Const c_sModuleFileName As String = "C:\MyJava\ARC-SDM\VBScripts\SpatialAnalystUtil.cls"

Public Function SetRasterWorkspace(sPath As String) As IWorkspace
  On Error GoTo ErrorHandler

    ' This function returns a raster workspace object given the path.
    ' The path needs to contain raster data.
    Dim pWSF As IWorkspaceFactory
11:     Set pWSF = New RasterWorkspaceFactory
12:     If pWSF.IsWorkspace(sPath) Then
13:         Set SetRasterWorkspace = pWSF.OpenFromFile(sPath, 0)
14:     End If
    Exit Function


  Exit Function
ErrorHandler:
20:    Set SetRasterWorkspace = Nothing
  HandleError True, "SetRasterWorkspace " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function SetFeatureShapeWorkspace(sPath As String) As IWorkspace
  On Error GoTo ErrorHandler

    ' This function returns a shapefile workspace object  given the path.
    ' The path needs to contain shape files
    Dim pWSF As IWorkspaceFactory
29:     Set pWSF = New ShapefileWorkspaceFactory
30:     If pWSF.IsWorkspace(sPath) Then
31:         Set SetFeatureShapeWorkspace = pWSF.OpenFromFile(sPath, 0)
32:     End If
  Exit Function
ErrorHandler:
  HandleError True, "SetFeatureShapeWorkspace " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function SetFeatureCoverageWorkspace(sPath As String) As IWorkspace
  On Error GoTo ErrorHandler

    ' This function returns a coverage workspace object given the path.
    ' The path needs to contain coverages or shape files.
    Dim pWSF As IWorkspaceFactory
43:     Set pWSF = New ArcInfoWorkspaceFactory
    Dim pWS As IWorkspace
45:     If pWSF.IsWorkspace(sPath) Then
46:         Set pWS = pWSF.OpenFromFile(sPath, 0)
47:         Set SetFeatureCoverageWorkspace = pWS
48:     End If


  Exit Function
ErrorHandler:
53:     Set SetFeatureCoverageWorkspace = Nothing
  HandleError True, "SetFeatureCoverageWorkspace " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function SetFeatureAccessWorkspace(DBPath As String) As IWorkspace
  On Error GoTo ErrorHandler

    ' This function returns a Access workspace object given the path.
    ' An example of a path would be "c:\temp\Test.mdb"
    Dim pWSF As IWorkspaceFactory
62:     Set pWSF = New AccessWorkspaceFactory
    Dim pPropSet As IPropertySet
64:     Set pPropSet = New PropertySet
65:     pPropSet.SetProperty "DATABASE", DBPath
66:     Set SetFeatureAccessWorkspace = pWSF.Open(pPropSet, 0)


  Exit Function
ErrorHandler:
71:     Set SetFeatureAccessWorkspace = Nothing
  HandleError True, "SetFeatureAccessWorkspace " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function SetSDEWorkspace(ByVal sServer As String, ByVal sInstance _
                As String, ByVal sUser As String, _
                ByVal sPassWD As String) As IWorkspace
  On Error GoTo ErrorHandler

    ' This function returns a SDE workspace given the connection parameters.
    Dim pWSF As IWorkspaceFactory
81:     Set pWSF = New SdeWorkspaceFactory
    Dim pPropSet As IPropertySet
83:     Set pPropSet = New PropertySet
84:     With pPropSet
85:         .SetProperty "Server", sServer
86:         .SetProperty "Instance", sInstance
'       .SetProperty "Database", "sde" ' Ignored with ArcSDE for Oracle
88:         .SetProperty "user", sUser
89:         .SetProperty "password", sPassWD
90:         .SetProperty "version", "sde.DEFAULT"
91:     End With
92:     Set SetSDEWorkspace = pWSF.Open(pPropSet, 0)
    Exit Function


  Exit Function
ErrorHandler:
98:   Set SetSDEWorkspace = Nothing
  HandleError True, "SetSDEWorkspace " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function SetSDEWorkspaceByConnectionFile(ByVal PathName As String) As IWorkspace
  On Error GoTo ErrorHandler

    ' This function returns a SDE workspace given the connection file.
    ' An example of connection file is 'C:\WINNT\Profiles\hongxu\Application Data\ESRI\ArcCatolog\Connection to server1.sde'
    Dim pWS As IWorkspace
    Dim pWsName As IWorkspaceName
108:     Set pWsName = New WorkspaceName
109:     pWsName.PathName = PathName
110:     pWsName.WorkspaceFactoryProgID = "esriDataSourcesGDB.SdeWorkspaceFactory"
    Dim pName As IName
112:     Set pName = pWsName
113:     Set pWS = pName.Open
  
115:     Set SetSDEWorkspaceByConnectionFile = pWS
    Exit Function


  Exit Function
ErrorHandler:
121:     Set SetSDEWorkspaceByConnectionFile = Nothing
  HandleError True, "SetSDEWorkspaceByConnectionFile " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function OpenRasterDataset(sPath As String, sRasterName As String) As IRasterDataset
  On Error GoTo ErrorHandler

    'Return RasterDataset Object given a file name and its directory
    Dim pWSFact As IWorkspaceFactory
    Dim pRasterWS As IRasterWorkspace
    
131:     Set pWSFact = New RasterWorkspaceFactory
132:     If pWSFact.IsWorkspace(sPath) Then
133:         Set pRasterWS = pWSFact.OpenFromFile(sPath, 0)
134:         Set OpenRasterDataset = pRasterWS.OpenRasterDataset(sRasterName)
135:     End If
    Exit Function


  Exit Function
ErrorHandler:
141:     Set OpenRasterDataset = Nothing
  HandleError True, "OpenRasterDataset " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function OpenFeatureClassFromShapefile(sPath As String, sShapeName As String) As IFeatureClass
  On Error GoTo ErrorHandler

    'Returns a FeatureClass object given a shapefile's name and it's path
    Dim pWSFact As IWorkspaceFactory
    Dim pFWS As IFeatureWorkspace

151:     Set pWSFact = New ShapefileWorkspaceFactory
152:     If pWSFact.IsWorkspace(sPath) Then
153:         Set pFWS = pWSFact.OpenFromFile(sPath, 0)
154:         Set OpenFeatureClassFromShapefile = pFWS.OpenFeatureClass(sShapeName)
155:     End If

  Exit Function
ErrorHandler:
159:     Set OpenFeatureClassFromShapefile = Nothing
  HandleError True, "OpenFeatureClassFromShapefile " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function OpenFeatureClassFromCoverage(sPath As String, sCovName As String, ClassID As Integer) As IFeatureClass
  On Error GoTo ErrorHandler

    ' Open a feature class in the specified coverage by giving a class ID
    ' Open a feature dataset
    Dim pFeatDs As IFeatureDataset
    Dim pFeatWs As IFeatureWorkspace
    Dim pWSF As IWorkspaceFactory
170:     Set pWSF = New ArcInfoWorkspaceFactory
171:     If pWSF.IsWorkspace(sPath) Then
172:         Set pFeatWs = pWSF.OpenFromFile(sPath, 0)
173:         Set pFeatDs = pFeatWs.OpenFeatureDataset(sCovName)
        ' Get a feature class from dataset
        Dim pFC As IFeatureClassContainer
176:         Set pFC = pFeatDs
177:         Set OpenFeatureClassFromCoverage = pFC.Class(ClassID)
178:     End If


  Exit Function
ErrorHandler:

184:     Set OpenFeatureClassFromCoverage = Nothing
  HandleError True, "OpenFeatureClassFromCoverage " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Sub ConvertToDescriptor(pInGDS As IGeoDataset, sWhereClause As String, _
           sSubFields As String, sInField As String)
  On Error GoTo ErrorHandler

    ' This function takes an input IGeoDataset, converts it to a
    ' raster or featureclass descriptor.
    ' Input will be unchanged if wrong type, empty where clause or fields occur.
    
195:     If Len(sWhereClause) + Len(sSubFields) = 0 Then
    ' Both fields and where clause are empty
197:         Err.Raise vbObjectError + 1, "SAUtil.SpatialAnalystUtil", "Both subfields and where clause are empty"
        Exit Sub
199:     End If
    
    ' create and fill the query filter
    Dim pFilt As IQueryFilter
203:     Set pFilt = New QueryFilter
204:     If Len(sWhereClause) > 0 Then pFilt.WhereClause = sWhereClause
205:     If Len(sSubFields) > 0 Then pFilt.SubFields = sSubFields
    
    ' Query filter prepared, now create and fill Descriptor if input acceptable type
    ' If not acceptable type (raster, featureclass), simply return existing geodataset
209:     If TypeOf pInGDS Is IRaster Then
        ' IRaster Pointer Input, use RasterDescriptor
        Dim pRDesc As IRasterDescriptor
212:         Set pRDesc = New RasterDescriptor
213:         pRDesc.Create pInGDS, pFilt, sInField
        ' Point the input value to the new descriptor and return
215:         Set pInGDS = pRDesc
        Exit Sub
217:     End If
    
219:     If TypeOf pInGDS Is IFeatureClass Then
        ' IFeatureClass Pointer Input, use FeatureClassDescriptor
        Dim pFCDesc As IFeatureClassDescriptor
222:         Set pFCDesc = New FeatureClassDescriptor
223:         pFCDesc.Create pInGDS, pFilt, sInField
        ' Point the input value to the new descriptor and return
225:         Set pInGDS = pFCDesc
        Exit Sub
227:     End If


  Exit Sub
ErrorHandler:
  HandleError True, "ConvertToDescriptor " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Sub
Public Function CreateRasterDescriptor(sPath As String, sRasterName As String, _
       sWhereClause As String, sSubField As String, sInField As String) As IRasterDescriptor
  On Error GoTo ErrorHandler

    ' This function returns a RasterDescriptor object. The inputs are a
    ' single band raster, a where clause and a list of fields as sub field
    ' "" can be used for where clause and subfield
241:     If Len(sWhereClause) + Len(sSubField) = 0 Then
    ' Both fields and where clause are empty
243:         Err.Raise vbObjectError + 1, "SAUtil.SpatialAnalystUtil", "Both subfields and where clause are empty"
        Exit Function
245:     End If
    
    Dim pDesc As IRasterDescriptor
248:     Set pDesc = New RasterDescriptor
    ' create and set query filter
    Dim pFilt As IQueryFilter
251:     Set pFilt = New QueryFilter
252:     If Len(sWhereClause) > 0 Then pFilt.WhereClause = sWhereClause
253:     If Len(sSubField) > 0 Then pFilt.SubFields = sSubField
    ' create raster from dataset
    Dim pRaster As IRaster
256:     Set pRaster = OpenRasterDataset(sPath, sRasterName).CreateDefaultRaster
    ' create RasterDescriptor
258:     pDesc.Create pRaster, pFilt, sInField
    
260:     Set CreateRasterDescriptor = pDesc
    Exit Function


  Exit Function
ErrorHandler:
266:     Set CreateRasterDescriptor = Nothing
  HandleError True, "CreateRasterDescriptor " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function CreateFeatureClassDescriptor(pFClass As IFeatureClass, _
    sWhereClause As String, sSubField As String, sInField As String) As IFeatureClassDescriptor
  On Error GoTo ErrorHandler

    ' This function returns a FeatureClassDescriptor. The inputs are a
    ' FeatureClass, a where clause and a list of fields as sub field
    ' "" can be used for where clause and subfield
276:     If Len(sWhereClause) + Len(sSubField) = 0 Then
    ' Both fields and where clause are empty
278:         Err.Raise vbObjectError + 1, "SAUtil.SpatialAnalystUtil", "Both subfields and where clause are empty"
        Exit Function
280:     End If
    
    Dim pDesc As IFeatureClassDescriptor
283:     Set pDesc = New FeatureClassDescriptor
    ' create and set query filter
    Dim pFilt As IQueryFilter
286:     Set pFilt = New QueryFilter
287:     If Len(sWhereClause) > 0 Then pFilt.WhereClause = sWhereClause
288:     If Len(sSubField) > 0 Then pFilt.SubFields = sSubField
    
290:     pDesc.Create pFClass, pFilt, sInField
    
292:     Set CreateFeatureClassDescriptor = pDesc
  Exit Function
ErrorHandler:
295:   Set CreateFeatureClassDescriptor = Nothing
  HandleError True, "CreateFeatureClassDescriptor " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Sub SetEnvironment(pOp As Variant, pExtent As IEnvelope, nCell As Double, _
                          pMask As IGeoDataset, sOutputPath As String, _
                          pPrj As ISpatialReference)
  On Error GoTo ErrorHandler

    ' Sets the environment of the input operator to the input extent, cellsize,
    ' mask, and Spatial Reference. Using "" or Nothing in the parameter to
    ' ignore the corespondent setting. The Output Path needs to contain raster data.

   ' Exit to avoid error if the passed variable is not a type of IRasterAnalysisEnvironment
   If Not (TypeOf pOp Is IRasterAnalysisEnvironment) Then Exit Sub
    
    ' Get an environment pointer to the Operator Object
    Dim pEnv As IRasterAnalysisEnvironment
312:     Set pEnv = pOp
    
    ' Set Extent
315:     If Not pExtent Is Nothing Then pEnv.SetExtent esriRasterEnvValue, pExtent
    
    ' Set cell size
318:     If nCell <> 0 Then pEnv.SetCellSize esriRasterEnvValue, nCell
    
    ' Set mask for analysis
321:     If Not pMask Is Nothing Then Set pEnv.Mask = pMask
    
    ' Set output spatial reference
324:     If Not pPrj Is Nothing Then Set pEnv.OutSpatialReference = pPrj
    
    ' Set output workspace
327:     If sOutputPath <> "" Then Set pEnv.OutWorkspace = SetRasterWorkspace(sOutputPath)
    Exit Sub


  Exit Sub
ErrorHandler:
  HandleError True, "SetEnvironment " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Sub
Public Function CreateSelectionSetFromShape(sPath As String, sShapeName As String, _
                sWhereClause As String, sSubField As String) As ISelectionSet
  On Error GoTo ErrorHandler

    ' This function creates a SelectionSet from a feature class
    
    ' One of the parameter has to be valid
    If Len(sWhereClause) + Len(sSubField) = 0 Then Exit Function
    Dim pTable As ITable
    Dim pFClass As IFeatureClass
    Dim pQF As IQueryFilter
    'Create and set QueryFilter
347:     Set pQF = New QueryFilter
348:     If Len(sWhereClause) > 0 Then pQF.WhereClause = sWhereClause
349:     If Len(sSubField) > 0 Then pQF.SubFields = sSubField
    ' Create FeatureClass and QI using ITable interface
351:     Set pFClass = OpenFeatureClassFromShapefile(sPath, sShapeName)
352:     Set pTable = pFClass
    ' Create a scratch workspace for tempory remap file
    Dim pSwf As IScratchWorkspaceFactory
355:     Set pSwf = New ScratchWorkspaceFactory
    Dim pSws As IWorkspace
357:     Set pSws = pSwf.CreateNewScratchWorkspace
    ' Create SelectionSet from Table
359:     Set CreateSelectionSetFromShape = pTable.Select(pQF, esriSelectionTypeHybrid, _
                                      esriSelectionOptionNormal, pSws)
  Exit Function
ErrorHandler:
363:   Set CreateSelectionSetFromShape = Nothing
  HandleError True, "CreateSelectionSetFromShape " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function CreateSelectionSetFromRaster(sPath As String, sRasterName As String, _
                sWhereClause As String, sSubFields As String) As ISelectionSet
  On Error GoTo ErrorHandler

    'This function returns a SelectionSet from a raster
    
    ' one of the two parameters has to be valid
    If Len(sWhereClause) + Len(sSubFields) = 0 Then Exit Function
    
    Dim pTable As ITable
    Dim pRBand As IRasterBand
    Dim pRDS As IRasterDataset
    Dim pQF As IQueryFilter
    ' Create QueryFilter
380:     Set pQF = New QueryFilter
381:     If Len(sWhereClause) > 0 Then pQF.WhereClause = sWhereClause
382:     If Len(sSubFields) > 0 Then pQF.SubFields = sSubFields
    ' open RasterDataset and get the first band
384:     Set pRDS = OpenRasterDataset(sPath, sRasterName)
    Dim pRBDC As IRasterBandCollection
386:     Set pRBDC = pRDS
387:     Set pRBand = pRBDC.item(0)
    ' Get table from band
389:     Set pTable = pRBand.AttributeTable
    
    ' Create a scratch workspace for tempery remap file
    Dim pSwf As IScratchWorkspaceFactory
393:     Set pSwf = New ScratchWorkspaceFactory
    Dim pSws As IWorkspace
395:     Set pSws = pSwf.CreateNewScratchWorkspace

    ' Create selection set from raster table
398:     Set CreateSelectionSetFromRaster = pTable.Select(pQF, esriSelectionTypeHybrid, _
                            esriSelectionOptionNormal, pSws)
  Exit Function
ErrorHandler:
402:   Set CreateSelectionSetFromRaster = Nothing
  HandleError True, "CreateSelectionSetFromRaster " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function CreateSelectionSetFromGeoDS(pInGeoDS As IGeoDataset, _
                sWhereClause As String, sSubFields As String) As ISelectionSet
  On Error GoTo ErrorHandler

    'The input pInGeoDS could only be a Raster or a FeatureClass object. Use
    ' RasterDescriptor::Create if the SelectionSet is created from a raster; use
    ' FeatureClassDescriptor if it is created from a featureclass
    
    ' one of the variables has to be valid
    If Len(sWhereClause) + Len(sSubFields) = 0 Then Exit Function
    
    Dim pQF As IQueryFilter
    ' Create QueryFilter
418:     Set pQF = New QueryFilter
419:     If Len(sWhereClause) > 0 Then pQF.WhereClause = sWhereClause
420:     If Len(sSubFields) > 0 Then pQF.SubFields = sSubFields

    Dim pTable As ITable
423:     If TypeOf pInGeoDS Is IRaster Then
        'Get table from a raster
        Dim pBand As IRasterBand
        Dim pBandC As IRasterBandCollection
427:         Set pBandC = pInGeoDS
428:         Set pBand = pBandC.item(0)
429:         Set pTable = pBand.AttributeTable
430:     ElseIf TypeOf pInGeoDS Is IFeatureClass Then
        ' Get table from a featureclass
432:         Set pTable = pInGeoDS
433:     Else
        Exit Function
435:     End If
    
    ' Create a scratch workspace for tempery remap file
    Dim pSwf As IScratchWorkspaceFactory
439:     Set pSwf = New ScratchWorkspaceFactory
    Dim pSws As IWorkspace
441:     Set pSws = pSwf.CreateNewScratchWorkspace

    ' Create selection set of the table
444:     Set CreateSelectionSetFromGeoDS = pTable.Select(pQF, esriSelectionTypeHybrid, _
                            esriSelectionOptionNormal, pSws)

  Exit Function
ErrorHandler:
449:    Set CreateSelectionSetFromGeoDS = Nothing
  HandleError True, "CreateSelectionSetFromGeoDS " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function

Public Function CreatePointCollectionFromShapeFile(sPath As String, sShapeName As String) As IPointCollection
  On Error GoTo ErrorHandler

    ' This function creates a point collection from a point shape file
    
    Dim pPointC As IPointCollection
    ' Create a FeatureClass
    Dim pFeatClass As IFeatureClass
461:     Set pFeatClass = OpenFeatureClassFromShapefile(sPath, sShapeName)
    ' Calculate the number of features in the FeatureClass
    Dim FeatCount As Long
464:     FeatCount = pFeatClass.FeatureCount(Nothing)
    If FeatCount = 0 Then Exit Function
    ' Add those features to a point collection object
    Dim i As Integer
    Dim pFeat As IFeature
    Dim pGeometry As IGeometry
470:     Set pGeometry = New Point
471:     Set pPointC = New Multipoint
    Dim Pnt As IPoint
473:     For i = 0 To FeatCount - 1
474:         Set pFeat = pFeatClass.GetFeature(i)
475:         Set pGeometry = pFeat.Shape
476:         Set Pnt = pGeometry
477:         pPointC.AddPoint Pnt
478:         Set Pnt = Nothing
479:         Set pFeat = Nothing
480:         Set pGeometry = Nothing
481:     Next i
    'return the pointer
483:     Set CreatePointCollectionFromShapeFile = pPointC
   Exit Function
ErrorHandler:
486:   Set CreatePointCollectionFromShapeFile = Nothing
  HandleError True, "CreatePointCollectionFromShapeFile " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function CreateSpatialReferenceGCS(PrjCode As Long) As ISpatialReference
  On Error GoTo ErrorHandler

    ' To create the Geographic Coordinate System from PCS (Projected Cooridinate System) code
    Dim pSRS As ISpatialReferenceFactory2
494:     Set pSRS = New SpatialReferenceEnvironment
495:     Set CreateSpatialReferenceGCS = pSRS.CreateGeographicCoordinateSystem(PrjCode)
    Exit Function

  Exit Function
ErrorHandler:
500:     Set CreateSpatialReferenceGCS = Nothing

  HandleError True, "CreateSpatialReferenceGCS " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function CreateSpatialReference(PrjCode As Long) As ISpatialReference
  On Error GoTo ErrorHandler

    ' create Projected Cooridinate System from PCS (Projected Cooridinate System) code
    Dim pSRS As ISpatialReferenceFactory2
509:     Set pSRS = New SpatialReferenceEnvironment
510:     Set CreateSpatialReference = pSRS.CreateSpatialReference(PrjCode)
    Exit Function
512: Set CreateSpatialReference = Nothing


  Exit Function
ErrorHandler:
517:     Set CreateSpatialReference = Nothing

  HandleError True, "CreateSpatialReference " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function

Public Function OpenTableFromFile(sPath As String, sTableName As String) As ITable
  On Error GoTo ErrorHandler

    ' This function returns an ITable interface pointer from an INFO or DBF file. it will give an
    ' automation error if table type is not a dbf table or info table
    
    Dim pTable As ITable
    Dim pWSFact As IWorkspaceFactory
    Dim pFeatWork As IFeatureWorkspace
    
    ' choice of workspace factory depends if table is INFO or DBF
    Dim sSplit() As String
534:     sSplit = Split(sTableName, ".")
535:     If UBound(sSplit) = LBound(sSplit) Then ' no dots in name, assume info table
536:         Set pWSFact = New ArcInfoWorkspaceFactory
537:     Else
538:         If sSplit(1) = "dbf" Then
539:             Set pWSFact = New ShapefileWorkspaceFactory
540:         Else    'not dbf assume info table
541:             Set pWSFact = New ArcInfoWorkspaceFactory
542:         End If
543:     End If
544:     If pWSFact.IsWorkspace(sPath) Then
545:         Set pFeatWork = pWSFact.OpenFromFile(sPath, 0)
546:         Set pTable = pFeatWork.OpenTable(sTableName)
547:     End If
        
    ' Return the table pointer
550:     Set OpenTableFromFile = pTable
    Exit Function


  Exit Function
ErrorHandler:
556:     Set OpenTableFromFile = Nothing
  HandleError True, "OpenTableFromFile " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function
Public Function OpenRasterTable(sPath As String, sFileName As String) As ITable
  On Error GoTo ErrorHandler

    ' this function will return an ITable interface pointer from a raster file.
    ' If it is a multiband image, it will return the table of the first
    ' band. If the input raster does not have a table, it will
    ' return Nothing
    Dim pRBandC As IRasterBandCollection
    Dim pBand As IRasterBand

569:     Set pRBandC = OpenRasterDataset(sPath, sFileName)
570:     Set pBand = pRBandC.item(0)
    ' get table and return the pointer
572:     Set OpenRasterTable = pBand.AttributeTable
    Exit Function

  Exit Function
ErrorHandler:
577:     Set OpenRasterTable = Nothing

  HandleError True, "OpenRasterTable " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function

Public Sub CheckSpatialAnalystLicense()
  On Error GoTo ErrorHandler

    ' This module is used to check in the SpatialAnalyst license
    ' in a standalone VB application.
    
    ' Get Spatial Analyst Extension UID
    Dim pUID As New UID
590:     pUID.Value = "esriSpatialAnalystUI.SAExtension.1"
    
    ' Add Spatial Analyst extension to the license manager
    Dim v As Variant
    Dim pLicAdmin As IExtensionManagerAdmin
595:     Set pLicAdmin = New ExtensionManager
596:     Call pLicAdmin.AddExtension(pUID, v)
    
    ' Enable the license
    Dim pLicManager As IExtensionManager
600:     Set pLicManager = pLicAdmin
    Dim pExtensionConfig As IExtensionConfig
602:     Set pExtensionConfig = pLicManager.FindExtension(pUID)
603:     pExtensionConfig.State = esriESEnabled
    Exit Sub


  Exit Sub
ErrorHandler:
609:     MsgBox "Failed in License Checking" & Err.Description
  HandleError True, "CheckSpatialAnalystLicense " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Sub
Public Function GetSpatialAnalystSettings(ByRef SDMApp As IApplication) As IRasterAnalysisEnvironment
  On Error GoTo ErrorHandler

    ' This function is used to get current Spatial Analyst's Settings:
    ' RasterAnalysis Object defined through Option dialog in Spatial
    ' Analyst UI, like workspace path, cell size, extent. However, it
    ' ignores the setting of output spatial reference in the option, in
    ' other words, it is not influenced by the change of setting
    ' outout spatial reference to be the same as the data frame's.
    Dim pExtension As IExtension
    'Dim pSASetting As ISpatialAnalysis ' Interface for Spatial Analyst Setting
        ' Find Spatial Analyst Extension
624:     Set pExtension = SDMApp.FindExtensionByName("Spatial Analyst")
625:     If Not pExtension Is Nothing Then
        ' QI IExtension for ISpatialAnalyst
                If TypeOf pExtension Is IExtension Then
627:         'Set pSASetting = pExtension
                    Set GetSpatialAnalystSettings = pExtension
                Else
                    Set GetSpatialAnalystSettings = Nothing
                    Exit Function
                End If
        ' Get IRasterAnalysisEnvironment
                Dim cellsize As Double
630:         GetSpatialAnalystSettings.GetCellSize esriRasterEnvValue, cellsize
631:         'Set GetSpatialAnalystSettings = pSASetting.AnalysisEnvironment
632:     Else
633:         Set GetSpatialAnalystSettings = Nothing
634:     End If
635:     Set pExtension = Nothing
  Exit Function
ErrorHandler:
639:     Set GetSpatialAnalystSettings = Nothing
  HandleError True, "GetSpatialAnalystSettings " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function

Public Function getApp() As IApplication
One:    On Error GoTo Three
161: Set getApp = New AppRef   'ArcMap App
Exit Function
Two:    On Error GoTo errh
'163: Set getApp = Application 'VBA App
'Exit Function
Three: On Error GoTo Two
166:    Set getApp = SDMGlobals.SDMDoc.Parent 'StandAlone App
Exit Function
errh:
169:     MsgBox "SDMUtils can't Initialize Application", , "SDMUtils"

End Function





